/*
 * JRainWords 0.2
 *
 * Useful (?) educational (??) tool (???).
 *
 * (C) 2001 Davide Alberani <alberanid@bigfoot.com>
 *          http://digilander.iol.it/alberanid/
 *
 * This code is released under the GPL license.
 *
 */


import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.text.*;
import javax.swing.event.*;
import java.applet.AudioClip;
import java.applet.Applet;

import java.io.*;
import java.util.*;
import java.net.URL;
import javax.swing.filechooser.*;



// The main class
public class Jrw extends JPanel {

    // program name, version number and other infos
    private final static String JRW_NAME = "JRainWords";
    private final static String JRW_VERSION = "0.2";
    private final static String JRW_LICENSE = "GPL";
    private final static String JRW_YEAR = "2001";
    private final static String JRW_AUTHOR = "Davide Alberani";
    private final static String JRW_EMAIL = "alberanid@bigfoot.com";
    private final static String JRW_HOMEPAGE =
                                    "http://digilander.iol.it/alberanid/";

    // the board
    private static EditBoard editor;
    // buttons bar
    private JToolBar toolbar;
    // menus
    private JMenuBar menubar;
    // the label in the statusbar
    private JLabel textlabel = new JLabel();

    // to choose a word, a color, etc.
    // XXX: initialize the seed?
    private Random rndgen = new Random();

    // configuration file.
    private final static String jrwDirName = ".jrw";
    private final static String jrwConfFileName = "jrw.cfg";
    private static File confFile;

    // some dialogs
    private JFileChooser fileChooser;
    private JDialog wordsDialog;
    private JDialog optionsDialog;
    private JOptionPane messagesDialog = new JOptionPane();

    // XXX: use a Vector and put everything in a file + ColorChooser
    private Color[] colorsList = {
        Color.blue,
        Color.green,
        Color.red,
        Color.magenta,
        Color.orange,
        new Color(100, 0, 150),
        new Color(35, 100, 80),
        new Color(160, 80, 50),
        new Color(160, 80, 255),
        new Color(255, 70, 255)
    };

    // default properties, and options read from user's file
    private static Properties defaultProperties = new Properties();
    private Properties options = new Properties();
    static {
        defaultProperties.setProperty("wordsfile", "jrwwords.txt");
        defaultProperties.setProperty("fontsize", "16");
        defaultProperties.setProperty("soundfile", "bravo.wav");
        defaultProperties.setProperty("windowheight", "480");
        defaultProperties.setProperty("windowwidth", "640");
        defaultProperties.setProperty("maxlength", "unlimited");
    }

    // the list of words from the external file
    private Vector wordsList = new Vector();

    // the word you have to guess
    private String currentWord;

    // max length when choosing a new word.  -1 is unlimited
    private int maxLength = -1;

    // default window size
    private static int wh = 480;
    private static int ww = 640;

    // commands for buttons and menu items
    private Hashtable commands;

    // list of actions defined in this panel
    private Action[] defaultActions = {
        new ExitAction(),
        new NewAction(),
        new ChangeWordAction(),
        new SaveAction(),   
        new OpenAction(),   
        new ModifyAction(),
        new ShowOptionsAction(),
        new AboutAction()
    };  
    
    // resources from the outher space
    private static ResourceBundle resources;
    // get the resources
    static {
        try {
            resources = ResourceBundle.getBundle("resources.Jrw",
                                                 Locale.getDefault());
        } catch (MissingResourceException mre) {
            System.err.println("Unable to read resources/Jrw.properties");
            System.exit(1);
        }
    }


    // that's me!
    public Jrw() {
        super(true);
        initUserOptions();
        initWords();
        try {
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
        } catch (Exception exc) {
            System.err.println("Error loading Look&Feel: " + exc);
        }
        setBorder(BorderFactory.createEtchedBorder());
        setLayout(new BorderLayout());
        // the edit board
        StyleContext sc = new StyleContext();
        DefaultStyledDocument doc = new DefaultStyledDocument(sc);
        editor = new EditBoard(doc, sc);
        commands = new Hashtable();
        Action[] actions = getActions();
        for (int i = 0; i < actions.length; i++) {
            Action a = actions[i];
            commands.put(a.getValue(Action.NAME), a);
        }
        JScrollPane scroller = new JScrollPane();
        scroller.setHorizontalScrollBarPolicy(
                    JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        JViewport port = scroller.getViewport();
        port.add(editor);
        JPanel panel = new JPanel();
        panel.setLayout(new BorderLayout());
        JPanel statusbar = new JPanel();
        statusbar.add(textlabel);
        createMenubar();
        add("North", menubar);
        panel.add("North", createToolbar());
        panel.add("Center", scroller);
        add("Center", panel);
        add("South", statusbar);
    }


    // oh, well...
    public static void main(String[] args) {
        try {
            JFrame frame = new JFrame();
            frame.setTitle(JRW_NAME + " " + JRW_VERSION);
            frame.setBackground(Color.lightGray);
            frame.getContentPane().setLayout(new BorderLayout());
            frame.getContentPane().add("Center", new Jrw());
            frame.addWindowListener(new AppCloser());
            frame.pack();
            frame.setSize(ww, wh);
            // it's nice, isn't it?
            frame.show();
        } catch (Throwable t) {
            System.err.println("Shit happens: " + t);
            t.printStackTrace();
        }
        getEditor().requestFocus();
    }


    // what to do on exit
    private static final class AppCloser extends WindowAdapter {
        public void windowClosing(WindowEvent e) {
            // XXX: ask?
            System.exit(0);
        }
    }


    // the board you write into
    private class EditBoard extends JTextPane implements KeyListener {

        DefaultStyledDocument doc;
        StyleContext styles;
        // there is new text in the board
        boolean modified = false;

        public EditBoard(DefaultStyledDocument d, StyleContext sc) {
            addKeyListener(this);
            initDocument(d, sc);
        }
        public void initDocument(DefaultStyledDocument d, StyleContext sc) {
            this.doc = d;
            this.styles = sc;
            setStyledDocument(doc);
            setFontSize();
            modified = false;
        }
        public void setFontSize() {
            Style s = styles.addStyle(null, null);
            int fs = Integer.parseInt(options.getProperty("fontsize", "16"));
            if (fs <= 0) fs = 16;
            StyleConstants.setFontSize(s, fs);
            doc.setLogicalStyle(doc.getEndPosition().getOffset(), s);
        }
        public boolean wasModified() {
            return modified;
        }
        public void setModified(boolean m) {
            modified = m;
        }
        public void keyTyped(KeyEvent e) {
        }
        public void keyPressed(KeyEvent e) {
        }
        // checks performed every time a key is released
        // XXX: optimize for speed
        public void keyReleased(KeyEvent e) {
            char thechar = e.getKeyChar();
            if (thechar == ' ' || thechar == '\t' ||
                thechar == '\n' || thechar == '\r' ||
                thechar == '\f' || thechar == KeyEvent.CHAR_UNDEFINED) {
                    return;
            }
            setModified(true);
            String txt = "";
            String[] txttok = null;
            int caretpos = getCaretPosition();
            try {
                txt = getText(0, caretpos);
            } catch (BadLocationException ble) {
                messagesDialog.showMessageDialog(getFrame(),
                            "No caret position",
                            "Warning",
                            JOptionPane.ERROR_MESSAGE);
                System.exit(20);
            }
            txttok = tokenize(txt);
            if (txttok.length == 0) {
                return;
            }
            String lastword = txttok[txttok.length - 1];
            // got a match
            if (lastword.equalsIgnoreCase(currentWord)) {
                gotcha(caretpos, lastword);
            }
        }
        // wow, good shot!
        public void gotcha(int cp, String s) {
                // change the color for the matching word
                int len = s.length();
                Style st = styles.addStyle(null, getLogicalStyle());
                int r = rndgen.nextInt(colorsList.length);
                StyleConstants.setForeground(st, colorsList[r]);
                doc.setCharacterAttributes(cp - len,
                                            len, st, true);
                // sing! ;-)
                playOnSuccess();
                // select another word
                selectRandomWord();
        }
    }


    // XXX: replace with a single function
    private void initUserOptions() {
        findConfFile();
        loadOptions();
    }


    // read options from the config file
    private void loadOptions() {
        try {
            if (confFile.isFile() && confFile.canRead()) {
                FileInputStream fstrm = new FileInputStream(confFile);
                options.load(fstrm);
                fstrm.close();
            } else {
                // cannot get a config file; goes on using the defaults
                options = defaultProperties;
            }
        } catch (FileNotFoundException fnfe) {
            messagesDialog.showMessageDialog(getFrame(),
                            "Cannot find options file.",
                            "Warning",
                            JOptionPane.WARNING_MESSAGE);
        } catch (IOException ioe) {
            messagesDialog.showMessageDialog(getFrame(),
                            "Cannot read options file.",
                            "Warning",
                            JOptionPane.WARNING_MESSAGE);
        }
        // time to set window's size
        ww=(new Integer(options.getProperty("windowwidth", "640"))).intValue();
        wh=(new Integer(options.getProperty("windowheight", "480"))).intValue();
    }


    // XXX: better algo
    private void findConfFile() {
        String fsep = System.getProperty("file.separator");
        File f;
        String userHome = System.getProperty("user.home");
        if (userHome != "") {
            // there is a user's home directory
            File homeDir = new File(userHome + fsep + jrwDirName);
            if (!homeDir.isDirectory()) {
                // there isn't the .jrw dir in the home
                homeDir.mkdirs();
                f = new File(userHome + fsep + jrwDirName + fsep +
                                jrwConfFileName);
                saveOptions(defaultProperties, f);
                confFile = f;
                messagesDialog.showMessageDialog(getFrame(),
                            "Configuration file created.",
                            "Warning",
                            JOptionPane.INFORMATION_MESSAGE);
                return;
            } else {
                // the directory ~/.jrw exists...
                f = new File(userHome + fsep + jrwDirName + fsep +
                                jrwConfFileName);
                if (f.isFile() && f.canRead()) {
                    // ...and even the configuration file
                    confFile = f;
                    return;
                } else {
                    // ...but not the configuration file
                    saveOptions(defaultProperties, f);
                    confFile = f;
                    messagesDialog.showMessageDialog(getFrame(),
                            "Configuration file created.",
                            "Warning",
                            JOptionPane.INFORMATION_MESSAGE);
                    return;
                }
            }
        }
        // unable to find a user home directory
        f = new File(jrwConfFileName);
        if (f.isFile() && f.canRead()) {
            // there is a configuration file in the current directory
            confFile = new File(jrwConfFileName);
            return;
        } else {
            // try creating a configuration file in this dir
            saveOptions(defaultProperties, f);
            confFile = f;
            messagesDialog.showMessageDialog(getFrame(),
                            "Configuration file created.",
                            "Warning",
                            JOptionPane.INFORMATION_MESSAGE);
        }
        // unable to get the configuration file
        messagesDialog.showMessageDialog(getFrame(),
                            "Unable to create a configuration file.",
                            "Warning",
                            JOptionPane.WARNING_MESSAGE);
    }


    // save current set of options
    private void saveOptions(Properties p, File f) {
        try {
            FileOutputStream fstrm = new FileOutputStream(f);
            p.store(fstrm, "Self generated file.  Do not edit!");
            fstrm.close();
        } catch (FileNotFoundException fnfe) {
            messagesDialog.showMessageDialog(getFrame(),
                            "Unable to create the configuration file.",
                            "Warning",
                            JOptionPane.WARNING_MESSAGE);
        } catch (IOException ioe) {
            messagesDialog.showMessageDialog(getFrame(),
                            "Unable to access the configuration file.",
                            "Warning",
                            JOptionPane.WARNING_MESSAGE);

        }
    }


    // exclude unfair words
    private boolean checkWord(String s) {
        if (s != null && !s.equals("") &&
            (s.indexOf(' ') == -1) &&
            (s.indexOf('\n') == -1) &&
            (s.indexOf('\t') == -1) &&
            (s.indexOf('\r') == -1) &&
            (s.indexOf('\f') == -1)) {
            return true;
        }
        return false;
    }


    // initialize the words list
    private void initWords() {
        readWordsList();
        selectRandomWord();
    }


    // read the file with the words list.
    private void readWordsList() {
        wordsList.clear();
        try {
            String txt;
            BufferedReader br = new BufferedReader(
                                    new FileReader(
                                            options.getProperty("wordsfile")));
            while (br.ready()) {
                txt = br.readLine();
                if (checkWord(txt)) {
                    wordsList.add(txt);
                }
            }
            br.close();
        } catch (FileNotFoundException fnfe) {
            messagesDialog.showMessageDialog(getFrame(),
                            "Cannot find words list.",
                            "Warning",
                            JOptionPane.WARNING_MESSAGE);
        } catch (IOException ioe) {
            messagesDialog.showMessageDialog(getFrame(),
                            "Unable to read words list.",
                            "Warning",
                            JOptionPane.WARNING_MESSAGE);
        }
    }


    // pick random words from the list
    // XXX: optimize for speed, be aware that other
    // methods for select a word will be available.
    private void selectRandomWord() {
        Vector tmpv = new Vector();
        int i = 0;
        if (maxLength == -1) {
            tmpv = (Vector) wordsList.clone();
        } else {
            while (i < wordsList.size()) {
                String elem = (String) wordsList.get(i);
                if (elem.length() <= maxLength) {
                    tmpv.add(elem);
                }
                i++;
            }
        }
        int ws = tmpv.size();
        if (ws > 0)  {
            int rn = rndgen.nextInt(tmpv.size());
            currentWord = (String) tmpv.get(rn);
        } else {
            currentWord = "";
            messagesDialog.showMessageDialog(getFrame(),
                            "Cannot select any word.",
                            "Warning",
                            JOptionPane.WARNING_MESSAGE);

        }
        textlabel.setText(getResourceString("write-this") +
                            currentWord.toUpperCase());
    }


    // play a wave file.
    private void playOnSuccess() {
        Applet app = new Applet();
        URL purl = null;
        try {
            purl = new URL("file:" + options.getProperty("soundfile"));
        } catch (java.net.MalformedURLException mue) {
            System.err.println("Unable to find the URL.");
        }
        AudioClip acp = app.newAudioClip(purl);
        acp.play();
    }


    // save the current set of words to the file
    private void saveWordsToFile() {
        try {
            BufferedWriter bw = new BufferedWriter(
                                    new FileWriter(
                                        options.getProperty("wordsfile")));
            int i = 0;
            String txt = "";
            while (i < wordsList.size()) {
                txt = (String) wordsList.get(i);
                bw.write(txt, 0, txt.length());
                bw.newLine();
                i++;
            }
            bw.flush();
            bw.close();
        } catch (IOException ioe) {
            messagesDialog.showMessageDialog(getFrame(),
                            "Unable to save words file.",
                            "Warning",
                            JOptionPane.WARNING_MESSAGE);
        }
    }


    // show the options panel
    private class ShowOptionsAction extends AbstractAction {

        ShowOptionsAction() {
            super("show-options");
        }
        ShowOptionsAction(String nm) {
            super(nm);
        }
        public void actionPerformed(ActionEvent e) {
            showOptionsDialog();
        }
    }


    // exit via button or menu
    private class ExitAction extends AbstractAction {

        ExitAction() {
            super("exit");
        }
        ExitAction(String nm) {
            super(nm);
        }
        public void actionPerformed(ActionEvent e) {
            int ret = askSaveDialog();
            switch (ret) {
                case 0:
                    showSaveDialog();
                    break;
                case 2:
                    break;
                default:
                    System.exit(0);
            }
        }
    }


    // a brand new board
    private class NewAction extends AbstractAction {

        NewAction() {
            super("new");
        }
        NewAction(String nm) {
            super(nm);
        }
        public void actionPerformed(ActionEvent e) {
            int ret = askSaveDialog();
            switch (ret) {
                case 0:
                    showSaveDialog();
                    return;
                case 2:
                    return;
            }
            StyleContext sc = new StyleContext();
            DefaultStyledDocument doc = new DefaultStyledDocument(sc);
            getEditor().initDocument(doc, sc);
            validate();
        }
    }


    // Change the current word
    private class ChangeWordAction extends AbstractAction {

        ChangeWordAction() {
            super("change-word");
        }
        ChangeWordAction(String nm) {
            super(nm);
        }
        public void actionPerformed(ActionEvent e) {
            selectRandomWord();
        }
    }


    // modify the words' list
    private class ModifyAction extends AbstractAction {
        ModifyAction() {
            super("modify-words");
        }
        ModifyAction(String nm) {
            super(nm);
        }
        public void actionPerformed(ActionEvent e) {
            showModifyWords();
        }
    }


    // open a file with a saved document
    private class OpenAction extends AbstractAction {
        OpenAction() {
            super("open");
        }
        OpenAction(String nm) {
            super(nm);
        }
        public void actionPerformed(ActionEvent e) {
            showOpenDialog();
        }
    }


    // Save the current document
    private class SaveAction extends AbstractAction {
        SaveAction() {
            super("save");
        }           
        SaveAction(String nm) {
            super(nm);
        }               
        public void actionPerformed(ActionEvent e) {
            showSaveDialog();
        }           
    }               


    // show the about dialog
    private class AboutAction extends AbstractAction {
        AboutAction() {
            super("about");
        }
        AboutAction(String nm) {
            super(nm);
        }
        public void actionPerformed(ActionEvent e) {
            showAboutDialog();
        }
    }


    // save this doc?
    // return: -1 if no modified, 0 if "ok", 1 if "no", 2 if "cancel"
    private int askSaveDialog() {
        int ret = -1;
        if (getEditor().wasModified()) {
            ret = messagesDialog.showConfirmDialog(getFrame(),
                        "Unsaved board.\nSave it now?",
                        "Save",
                        JOptionPane.YES_NO_CANCEL_OPTION,
                        JOptionPane.QUESTION_MESSAGE);
        }
        return ret;
    }


    // modify the list of words
    private void showModifyWords() {
        Frame frame = getFrame();
        if (wordsDialog == null) {
            JPanel p = new JPanel();
            wordsDialog = new JDialog(frame, "Words", true);
            Box wlbox = new Box(BoxLayout.Y_AXIS);
            JButton rmb = new JButton("Remove Word");
            final DefaultListModel dsm = new DefaultListModel();
            int i = 0;
            while (i < wordsList.size()) {
                dsm.addElement(wordsList.get(i));
                i++;
            }
            final JList jl = new JList(dsm);
            rmb.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    int ri = jl.getSelectedIndex();
                    if (ri != -1) {
                        dsm.remove(ri);
                    }
                }
            });
            JButton rs = new JButton("Save");
            rs.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    int i = 0;
                    wordsList.clear();
                    while (i < dsm.getSize()) {
                        wordsList.add(dsm.getElementAt(i));
                        i++;
                    }
                    saveWordsToFile();
                    wordsDialog.setVisible(false);
                    getEditor().requestFocus();
                }
            });
            JButton ok = new JButton("Ok");
            ok.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    int i = 0;
                    wordsList.clear();
                    while (i < dsm.getSize()) {
                        wordsList.add(dsm.getElementAt(i));
                        i++;
                    }
                    wordsDialog.setVisible(false);
                    getEditor().requestFocus();
                }
            });
            final JTextField tf = new JTextField(15);
            JPanel tfp = new JPanel();
            tfp.setLayout(new BorderLayout(1, 1));
            p.setLayout(new BorderLayout(10, 10));
            JButton addb = new JButton("Add this Word");
            addb.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    String wtxt = tf.getText();
                    if (checkWord(wtxt)) {
                        dsm.addElement(wtxt);
                        tf.setText("");
                    } else {
                        messagesDialog.showMessageDialog(getFrame(),
                            "Bad chars in the word.",
                            "Warning",
                            JOptionPane.WARNING_MESSAGE);

                    }
                }
            });
            JScrollPane sp = new JScrollPane(jl);
            wlbox.add(sp);
            wlbox.add(rmb);
            tfp.add(tf, BorderLayout.WEST);
            tfp.add(addb, BorderLayout.EAST);
            Box buttonbox = new Box(BoxLayout.X_AXIS);
            buttonbox.add(ok);
            buttonbox.add(rs);
            wordsDialog.getContentPane().add(wlbox, BorderLayout.CENTER);
            wordsDialog.getContentPane().add(tfp, BorderLayout.NORTH);
            wordsDialog.getContentPane().add(buttonbox, BorderLayout.SOUTH);
            wordsDialog.pack();
        }
        wordsDialog.show();
    }


    // just a moment to myself ;-)
    private void showAboutDialog() {
        messagesDialog.showMessageDialog(getFrame(),
                        JRW_NAME + " " + JRW_VERSION + "\n" +
                        "A simple educational tool\n" +
                        "under " + JRW_LICENSE + " license.\n\n" +
                        "(C) " + JRW_YEAR + " " + JRW_AUTHOR + "\n" +
                        JRW_EMAIL + "\n" +
                        JRW_HOMEPAGE,
                        "About " + JRW_NAME,
                        JOptionPane.INFORMATION_MESSAGE);
    }


    // options panel
    private void showOptionsDialog() {
        Frame frame = getFrame();
        if (optionsDialog == null) {
            optionsDialog = new JDialog(frame, "Options", true);
            JPanel p = new JPanel();
            p.setLayout(new BorderLayout());
            JPanel centerp = new JPanel();
            centerp.setLayout(new BoxLayout(centerp, BoxLayout.X_AXIS));
            JPanel centerpr = new JPanel();
            centerpr.setLayout(new BoxLayout(centerpr, BoxLayout.Y_AXIS));
            JPanel centerpl = new JPanel();
            centerpl.setLayout(new BoxLayout(centerpl, BoxLayout.Y_AXIS));
            final JComboBox cp = new JComboBox();
            JLabel lab = new JLabel ("Select maximum length of the words:");
            Box lenbox = new Box(BoxLayout.X_AXIS);
            cp.addItem("4");
            cp.addItem("5");
            cp.addItem("6");
            cp.addItem("7");
            cp.addItem("8");
            cp.addItem("unlimited");
            cp.setSelectedItem(options.getProperty("maxlength"));
            lenbox.add(lab);
            lenbox.add(cp);
            Box topbox = new Box(BoxLayout.Y_AXIS);
            topbox.add(lenbox);
            JLabel fsl = new JLabel("Font size:");
            final JComboBox fsv = new JComboBox();
            fsv.addItem("8");
            fsv.addItem("10");
            fsv.addItem("12");
            fsv.addItem("14");
            fsv.addItem("16");
            fsv.addItem("18");
            fsv.addItem("22");
            fsv.addItem("24");
            fsv.addItem("26");
            fsv.setSelectedItem(options.getProperty("fontsize"));
            Box fsbox = new Box(BoxLayout.X_AXIS);
            fsbox.add(fsl);
            fsbox.add(fsv);
            topbox.add(fsbox);
            p.add(topbox, BorderLayout.NORTH);
            JLabel whl = new JLabel("Window height:");
            final JTextField whv = new JTextField(4);
            whv.setText(options.getProperty("windowheight"));
            Box whbox = new Box(BoxLayout.X_AXIS);
            whbox.add(whl);
            whbox.add(whv);
            centerpl.add(whbox);
            JLabel wwl = new JLabel("Window width:");
            final JTextField wwv = new JTextField(4);
            wwv.setText(options.getProperty("windowwidth"));
            Box wwbox = new Box(BoxLayout.X_AXIS);
            wwbox.add(wwl);
            wwbox.add(wwv);
            centerpl.add(wwbox);
            JLabel ftl = new JLabel("Words file:");
            final JTextField ftv = new JTextField(20);
            ftv.setText(options.getProperty("wordsfile"));
            JButton ftb = new JButton("..");
            ftb.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    Frame frame = getFrame();
                    if (fileChooser == null) { 
                        fileChooser = new JFileChooser();
                    }       
                    fileChooser.showOpenDialog(frame);
                    File f = fileChooser.getSelectedFile();
                    if (f != null && f.exists() && f.canRead()) {
                        ftv.setText(f.getAbsolutePath());
                    }
                }
            });
            Box ftbox = new Box(BoxLayout.X_AXIS);
            ftbox.add(ftl);
            ftbox.add(ftv);
            ftbox.add(ftb);
            centerpr.add(ftbox);
            JLabel fwl = new JLabel("Wave file:");
            final JTextField fwv = new JTextField(20);
            fwv.setText(options.getProperty("soundfile"));
            JButton fwb = new JButton("..");
            fwb.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    Frame frame = getFrame();
                    if (fileChooser == null) {
                        fileChooser = new JFileChooser();
                    }
                    fileChooser.showOpenDialog(frame);
                    File f = fileChooser.getSelectedFile();
                    if (f != null && f.exists() && f.canRead()) {
                        fwv.setText(f.getAbsolutePath());
                    }
                }
            });
            Box fwbox = new Box(BoxLayout.X_AXIS);
            fwbox.add(fwl);
            fwbox.add(fwv);
            fwbox.add(fwb);
            centerpr.add(fwbox);
            centerp.add(centerpl);
            centerp.add(centerpr);
            p.add(centerp, BorderLayout.CENTER);
            JButton rs = new JButton("Save");
            rs.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    String ml = (String) cp.getSelectedItem();
                    String whval = whv.getText();
                    String wwval = wwv.getText();
                    String ftval = ftv.getText();
                    String fwval = fwv.getText();
                    String fsval = (String) fsv.getSelectedItem();
                    if (checkOptionsAndSave(ml, whval, wwval,
                        ftval, fwval, fsval)) {
                        optionsDialog.setVisible(false);
                        getEditor().requestFocus();
                    }
                }
            });
            JButton ok = new JButton("Ok");
            ok.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    String ml = (String) cp.getSelectedItem();
                    String whval = whv.getText();
                    String wwval = wwv.getText();
                    String ftval = ftv.getText();
                    String fwval = fwv.getText();
                    String fsval = (String) fsv.getSelectedItem();
                    if (checkOptions(ml, whval, wwval, ftval, fwval, fsval)) {
                        optionsDialog.setVisible(false);
                        getEditor().requestFocus();
                    }
                }
            });
            Box buttonbox = new Box(BoxLayout.X_AXIS);
            buttonbox.add(ok);
            buttonbox.add(rs);
            p.add(buttonbox, BorderLayout.SOUTH);
            optionsDialog.getContentPane().add(p);
            optionsDialog.pack();
        }
        optionsDialog.show();
    }


    // check options and - if all are valid - save
    private boolean checkOptionsAndSave(String ml, String whval, String wwval,
                                    String ftval, String fwval, String fsval) {
        boolean ret = checkOptions(ml, whval, wwval, ftval, fwval, fsval);
        if (ret) {
            saveOptions(options, confFile);
        }
        return ret;
    }


    // check the options
    private boolean checkOptions(String ml, String whval, String wwval,
                                String ftval, String fwval, String fsval) {
        boolean ret = true;
        int max = -1;
        try {
            max = (new Integer(ml)).intValue();
        } catch (NumberFormatException nfe) {}
        maxLength = max;
        int wh = 0;
        int ww = 0;
        try {
            wh = (new Integer(whval)).intValue();
            ww = (new Integer(wwval)).intValue();
        } catch (NumberFormatException nfe) {}
        if ((wh >= 100 && wh <= 1000) &&
            (ww >= 100 && ww <= 1000)) {
            options.setProperty("windowheight", whval);
            options.setProperty("windowwidth", wwval);
            getFrame().setSize(ww, wh);
        } else {
            messagesDialog.showMessageDialog(getFrame(),
                            "Window's size out of limits.",
                            "Warning",
                            JOptionPane.WARNING_MESSAGE);
            ret = false;
        }
        if (!ftval.equals(options.getProperty("wordsfile"))) {
            options.setProperty("wordsfile", ftval);
            initWords();
        }
        options.setProperty("soundfile", fwval);
        if (!fsval.equals(options.getProperty("fontsize"))) {
            options.setProperty("fontsize", fsval);
            getEditor().setFontSize();
        }
        return ret;
    }


    // open a new doc
    private void showOpenDialog() {
        int ret = askSaveDialog();
        switch (ret) {
            case 0:
                showSaveDialog();
                return;
            case 2:
                return;
        }
        Frame frame = getFrame();
        if (fileChooser == null) {
            fileChooser = new JFileChooser();
        }
        fileChooser.showOpenDialog(frame);
        File f = fileChooser.getSelectedFile();
        if (f != null && f.exists()) {
            try {
                FileInputStream fin = new FileInputStream(f);
                ObjectInputStream istrm = new ObjectInputStream(fin);
                DefaultStyledDocument doc =
                            (DefaultStyledDocument) istrm.readObject();;
                StyleContext sc = new StyleContext();
                getEditor().initDocument(doc, sc);
                validate();
            } catch (IOException io) {
                messagesDialog.showMessageDialog(getFrame(),
                        "Cannot read the file.",
                        "Warning",
                        JOptionPane.WARNING_MESSAGE);
            } catch (ClassNotFoundException cnf) {
                System.err.println("Class not found: " + cnf.getMessage());
            }
        } else {
            if (f != null) {
                messagesDialog.showMessageDialog(getFrame(),
                        "Cannot read the file.",
                        "Warning",
                        JOptionPane.WARNING_MESSAGE);
            }
        }
    }


    // saving the current doc
    private void showSaveDialog() {
        Frame frame = getFrame();
        if (fileChooser == null) {
            fileChooser = new JFileChooser();
        }
        fileChooser.showSaveDialog(frame);
        File f = fileChooser.getSelectedFile();
        if (f == null || f.getName() == "") {
            return;
        }
        try {
            FileOutputStream fstrm = new FileOutputStream(f);
            ObjectOutput ostrm = new ObjectOutputStream(fstrm);
            ostrm.writeObject(getEditor().getDocument());
            ostrm.flush();
        } catch (IOException io) {
            messagesDialog.showMessageDialog(getFrame(),
                        "Cannot write the file.",
                        "Warning",
                        JOptionPane.WARNING_MESSAGE);
            return;
        }
        getEditor().setModified(false);
    }


    // return the current frame
    private Frame getFrame() {
        for (Container p = getParent(); p != null; p = p.getParent()) {
            if (p instanceof Frame) {
                return (Frame) p;
            }
        }
        return null;
    }


    // this board
    private static EditBoard getEditor() {
        return  editor;
    }


    // fetch the command list for the editor
    private Action[] getActions() {
        return TextAction.augmentList(editor.getActions(), defaultActions);
    }


    // get the action for the named string
    private Action getAction(String cmd) {
        return (Action) commands.get(cmd);
    }
    

    // buid the menus
    private void createMenubar() {
        menubar = new JMenuBar();
        String[] menuKeys = tokenize(getResourceString("menubar"));
        for (int i = 0; i < menuKeys.length; i++) {
            JMenu m = createMenu(menuKeys[i]);
            menubar.add(m);
        }
    }


    // create a menu
    private JMenu createMenu(String key) {
        String[] itemKeys = tokenize(getResourceString(key));
        JMenu menu = new JMenu(getResourceString(key + "Label"));
        for (int i = 0; i < itemKeys.length; i++) {
            if (itemKeys[i].equals("-")) {
                menu.addSeparator();
            } else {
                JMenuItem mi = createMenuItem(itemKeys[i]);
                menu.add(mi);
            }
        }
        return menu;
    }


    // create an entry for a menu
    private JMenuItem createMenuItem(String cmd) {
        JMenuItem mi = new JMenuItem(getResourceString(cmd + "Label"));
        String astr = getResourceString(cmd + "Action");
        if (astr == null) {
            astr = cmd;
        }
        mi.setActionCommand(astr);
        Action a = getAction(astr);
        if (a != null) {
            mi.addActionListener(a);
        } else {
            mi.setEnabled(false);
        }
        return mi;
    }


    // create the button's bar
    private Component createToolbar() {
        toolbar = new JToolBar();
        String[] toolKeys = tokenize(getResourceString("toolbar"));
        for (int i = 0; i < toolKeys.length; i++) {
            if (toolKeys[i].equals("-")) {
                toolbar.add(Box.createHorizontalStrut(5));
            } else {
                toolbar.add(createToolbarButton(toolKeys[i]));
            }
        }
        toolbar.add(Box.createHorizontalGlue());
        return toolbar;
    }


    // create a single button
    private JButton createToolbarButton(String key) {
        URL url = getResource(key + "Image");
        JButton b = new JButton(new ImageIcon(url)) {
            public float getAlignmentY() { return 0.5f; }
        };
        b.setRequestFocusEnabled(false);
        b.setMargin(new Insets(1, 1, 1, 1));

        String astr = getResourceString(key + "Action");
        if (astr == null) {
            astr = key;
        }
        Action a = getAction(astr);
        if (a != null) {
            b.setActionCommand(astr);
            b.addActionListener(a);
        } else {
            b.setEnabled(false);
        }
        String tip = getResourceString(key + "Tooltip");
        if (tip != null) {
            b.setToolTipText(tip);
        }
        return b;
    }


    // got tokens from a string
    private String[] tokenize(String input) {
        Vector v = new Vector();
        StringTokenizer t = new StringTokenizer(input);
        String[] cmd;
        while (t.hasMoreTokens())
            v.addElement(t.nextToken());
        cmd = new String[v.size()];
        for (int i = 0; i < cmd.length; i++)
            cmd[i] = (String) v.elementAt(i);
        return cmd;
    }

    // read a resource
    private String getResourceString(String nm) {
        String str;
        try {
            str = resources.getString(nm);
        } catch (MissingResourceException mre) {
            str = null;
        }
        return str;
    }


    // return a path
    private URL getResource(String key) {
        String name = getResourceString(key);
        if (name != null) {
            URL url = this.getClass().getResource(name);
            return url;
        }
        return null;
    }

}


/*
 * EOF
 * Mario manda tutti a nanna e poi chiude il bar.
 *
 * Davide Alberani.
 *
 */

